@@ -462,6 +462,8 @@ private:
 inline
 QList<EffectWindow*> EffectsHandlerImpl::elevatedWindows() const
 {
+    if (isScreenLocked())
+        return QList<EffectWindow*>();
     return elevated_windows;
 }
 
