@@ -69,6 +69,14 @@ public:
      * @see geometryChanged()
      **/
     QRect geometry() const;
+    /**
+     * The output name of the screen (usually eg. LVDS-1, VGA-0 or DVI-I-1 etc.)
+     */
+    virtual QString name(int screen) const;
+    /**
+     * @returns current refreshrate of the @p screen.
+     **/
+    virtual float refreshRate(int screen) const;
     /**
      * @returns size of the @p screen.
      *
