@@ -1125,6 +1125,18 @@ public:
         }
         return xcb_randr_get_screen_resources_crtcs(data());
     }
+    inline xcb_randr_mode_info_t *modes() {
+        if (isNull()) {
+            return nullptr;
+        }
+        return xcb_randr_get_screen_resources_modes(data());
+    }
+    inline uint8_t *names() {
+        if (isNull()) {
+            return nullptr;
+        }
+        return xcb_randr_get_screen_resources_names(data());
+    }
 };
 
 XCB_WRAPPER_DATA(CrtcGammaData, xcb_randr_get_crtc_gamma, xcb_randr_crtc_t)
@@ -1159,6 +1171,30 @@ public:
         }
         return QRect(info->x, info->y, info->width, info->height);
     }
+    inline xcb_randr_output_t *outputs() {
+        const CrtcInfoData::reply_type *info = data();
+        if (!info || info->num_outputs == 0 || info->mode == XCB_NONE || info->status != XCB_RANDR_SET_CONFIG_SUCCESS) {
+            return nullptr;
+        }
+        return xcb_randr_get_crtc_info_outputs(info);
+    }
+};
+
+XCB_WRAPPER_DATA(OutputInfoData, xcb_randr_get_output_info, xcb_randr_output_t, xcb_timestamp_t)
+class OutputInfo : public Wrapper<OutputInfoData, xcb_randr_output_t, xcb_timestamp_t>
+{
+public:
+    OutputInfo() = default;
+    OutputInfo(const OutputInfo&) = default;
+    explicit OutputInfo(xcb_randr_output_t c, xcb_timestamp_t t) : Wrapper<OutputInfoData, xcb_randr_output_t, xcb_timestamp_t>(c, t) {}
+
+    inline QString name() {
+        const OutputInfoData::reply_type *info = data();
+        if (!info || info->num_crtcs == 0 || info->num_modes == 0 || info->status != XCB_RANDR_SET_CONFIG_SUCCESS) {
+            return QString();
+        }
+        return QString::fromUtf8(reinterpret_cast<char*>(xcb_randr_get_output_info_name(info)), info->name_len);
+    }
 };
 
 XCB_WRAPPER_DATA(CurrentResourcesData, xcb_randr_get_screen_resources_current, xcb_window_t)
@@ -1173,6 +1209,12 @@ public:
         }
         return xcb_randr_get_screen_resources_current_crtcs(data());
     }
+    inline xcb_randr_mode_info_t *modes() {
+        if (isNull()) {
+            return nullptr;
+        }
+        return xcb_randr_get_screen_resources_current_modes(data());
+    }
 };
 
 XCB_WRAPPER(SetCrtcConfig, xcb_randr_set_crtc_config, xcb_randr_crtc_t, xcb_timestamp_t, xcb_timestamp_t, int16_t, int16_t, xcb_randr_mode_t, uint16_t, uint32_t, const xcb_randr_output_t*)
